package org.apache.bcel.generic;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.generic.ClassGenException;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.IndexedInstruction;
import org.apache.bcel.generic.Instruction;
import org.apache.bcel.generic.Type;
import org.apache.bcel.generic.TypedInstruction;
import org.apache.bcel.util.ByteSequence;

public abstract class LocalVariableInstruction extends Instruction implements TypedInstruction, IndexedInstruction {
  @Deprecated
  protected int n = -1;
  
  private short cTag = -1;
  
  private short canonTag = -1;
  
  LocalVariableInstruction() {}
  
  LocalVariableInstruction(short canonTag, short cTag) {
    this.canonTag = canonTag;
    this.cTag = cTag;
  }
  
  protected LocalVariableInstruction(short opcode, short cTag, int n) {
    super(opcode, (short)2);
    this.cTag = cTag;
    this.canonTag = opcode;
    setIndex(n);
  }
  
  public void dump(DataOutputStream out) throws IOException {
    if (wide())
      out.writeByte(196); 
    out.writeByte(getOpcode());
    if (getLength() > 1)
      if (wide()) {
        out.writeShort(this.n);
      } else {
        out.writeByte(this.n);
      }  
  }
  
  public short getCanonicalTag() {
    return this.canonTag;
  }
  
  public final int getIndex() {
    return this.n;
  }
  
  public Type getType(ConstantPoolGen cp) {
    switch (this.canonTag) {
      case 21:
      case 54:
        return Type.INT;
      case 22:
      case 55:
        return Type.LONG;
      case 24:
      case 57:
        return Type.DOUBLE;
      case 23:
      case 56:
        return Type.FLOAT;
      case 25:
      case 58:
        return Type.OBJECT;
    } 
    throw new ClassGenException("Unknown case in switch" + this.canonTag);
  }
  
  protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
    if (wide) {
      this.n = bytes.readUnsignedShort();
      setLength(4);
    } else {
      short opcode = getOpcode();
      if ((opcode >= 21 && opcode <= 25) || (opcode >= 54 && opcode <= 58)) {
        this.n = bytes.readUnsignedByte();
        setLength(2);
      } else {
        if (opcode <= 45) {
          this.n = (opcode - 26) % 4;
        } else {
          this.n = (opcode - 59) % 4;
        } 
        setLength(1);
      } 
    } 
  }
  
  public void setIndex(int n) {
    if (n < 0 || n > 65535)
      throw new ClassGenException("Illegal value: " + n); 
    this.n = n;
    if (n <= 3) {
      setOpcode((short)(this.cTag + n));
      setLength(1);
    } else {
      setOpcode(this.canonTag);
      if (wide()) {
        setLength(4);
      } else {
        setLength(2);
      } 
    } 
  }
  
  final void setIndexOnly(int n) {
    this.n = n;
  }
  
  public String toString(boolean verbose) {
    short opcode = getOpcode();
    if ((opcode >= 26 && opcode <= 45) || (opcode >= 59 && opcode <= 78))
      return super.toString(verbose); 
    return super.toString(verbose) + " " + this.n;
  }
  
  private boolean wide() {
    return (this.n > 255);
  }
}
